from __future__ import annotations
from dataclasses import dataclass
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.response_handler import ResponseHandler
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from ....models import internet_explorer_mode
    from ....models.o_data_errors import o_data_error
    from .site_lists import site_lists_request_builder
    from .site_lists.item import browser_site_list_item_request_builder

class InternetExplorerModeRequestBuilder():
    """
    Provides operations to manage the internetExplorerMode property of the microsoft.graph.edge entity.
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Optional[Union[Dict[str, Any], str]] = None) -> None:
        """
        Instantiates a new InternetExplorerModeRequestBuilder and sets the default values.
        Args:
            pathParameters: The raw url or the Url template parameters for the request.
            requestAdapter: The request adapter to use to execute the requests.
        """
        if path_parameters is None:
            raise Exception("path_parameters cannot be undefined")
        if request_adapter is None:
            raise Exception("request_adapter cannot be undefined")
        # Url template to use to build the URL for the current request builder
        self.url_template: str = "{+baseurl}/admin/edge/internetExplorerMode{?%24select,%24expand}"

        url_tpl_params = get_path_parameters(path_parameters)
        self.path_parameters = url_tpl_params
        self.request_adapter = request_adapter
    
    async def delete(self,request_configuration: Optional[InternetExplorerModeRequestBuilderDeleteRequestConfiguration] = None) -> bytes:
        """
        Delete navigation property internetExplorerMode for admin
        Args:
            requestConfiguration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: bytes
        """
        request_info = self.to_delete_request_information(
            request_configuration
        )
        from ....models.o_data_errors import o_data_error

        error_mapping: Dict[str, ParsableFactory] = {
            "4XX": o_data_error.ODataError,
            "5XX": o_data_error.ODataError,
        }
        if not self.request_adapter:
            raise Exception("Http core is null") 
        return await self.request_adapter.send_primitive_async(request_info, "bytes", error_mapping)
    
    async def get(self,request_configuration: Optional[InternetExplorerModeRequestBuilderGetRequestConfiguration] = None) -> Optional[internet_explorer_mode.InternetExplorerMode]:
        """
        Get internetExplorerMode from admin
        Args:
            requestConfiguration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[internet_explorer_mode.InternetExplorerMode]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        from ....models.o_data_errors import o_data_error

        error_mapping: Dict[str, ParsableFactory] = {
            "4XX": o_data_error.ODataError,
            "5XX": o_data_error.ODataError,
        }
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from ....models import internet_explorer_mode

        return await self.request_adapter.send_async(request_info, internet_explorer_mode.InternetExplorerMode, error_mapping)
    
    async def patch(self,body: Optional[internet_explorer_mode.InternetExplorerMode] = None, request_configuration: Optional[InternetExplorerModeRequestBuilderPatchRequestConfiguration] = None) -> Optional[internet_explorer_mode.InternetExplorerMode]:
        """
        Update the navigation property internetExplorerMode in admin
        Args:
            body: The request body
            requestConfiguration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[internet_explorer_mode.InternetExplorerMode]
        """
        if body is None:
            raise Exception("body cannot be undefined")
        request_info = self.to_patch_request_information(
            body, request_configuration
        )
        from ....models.o_data_errors import o_data_error

        error_mapping: Dict[str, ParsableFactory] = {
            "4XX": o_data_error.ODataError,
            "5XX": o_data_error.ODataError,
        }
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from ....models import internet_explorer_mode

        return await self.request_adapter.send_async(request_info, internet_explorer_mode.InternetExplorerMode, error_mapping)
    
    def site_lists_by_id(self,id: str) -> browser_site_list_item_request_builder.BrowserSiteListItemRequestBuilder:
        """
        Provides operations to manage the siteLists property of the microsoft.graph.internetExplorerMode entity.
        Args:
            id: Unique identifier of the item
        Returns: browser_site_list_item_request_builder.BrowserSiteListItemRequestBuilder
        """
        if id is None:
            raise Exception("id cannot be undefined")
        from .site_lists.item import browser_site_list_item_request_builder

        url_tpl_params = get_path_parameters(self.path_parameters)
        url_tpl_params["browserSiteList%2Did"] = id
        return browser_site_list_item_request_builder.BrowserSiteListItemRequestBuilder(self.request_adapter, url_tpl_params)
    
    def to_delete_request_information(self,request_configuration: Optional[InternetExplorerModeRequestBuilderDeleteRequestConfiguration] = None) -> RequestInformation:
        """
        Delete navigation property internetExplorerMode for admin
        Args:
            requestConfiguration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation()
        request_info.url_template = self.url_template
        request_info.path_parameters = self.path_parameters
        request_info.http_method = Method.DELETE
        if request_configuration:
            request_info.add_request_headers(request_configuration.headers)
            request_info.add_request_options(request_configuration.options)
        return request_info
    
    def to_get_request_information(self,request_configuration: Optional[InternetExplorerModeRequestBuilderGetRequestConfiguration] = None) -> RequestInformation:
        """
        Get internetExplorerMode from admin
        Args:
            requestConfiguration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation()
        request_info.url_template = self.url_template
        request_info.path_parameters = self.path_parameters
        request_info.http_method = Method.GET
        request_info.headers["Accept"] = ["application/json"]
        if request_configuration:
            request_info.add_request_headers(request_configuration.headers)
            request_info.set_query_string_parameters_from_raw_object(request_configuration.query_parameters)
            request_info.add_request_options(request_configuration.options)
        return request_info
    
    def to_patch_request_information(self,body: Optional[internet_explorer_mode.InternetExplorerMode] = None, request_configuration: Optional[InternetExplorerModeRequestBuilderPatchRequestConfiguration] = None) -> RequestInformation:
        """
        Update the navigation property internetExplorerMode in admin
        Args:
            body: The request body
            requestConfiguration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        if body is None:
            raise Exception("body cannot be undefined")
        request_info = RequestInformation()
        request_info.url_template = self.url_template
        request_info.path_parameters = self.path_parameters
        request_info.http_method = Method.PATCH
        request_info.headers["Accept"] = ["application/json"]
        if request_configuration:
            request_info.add_request_headers(request_configuration.headers)
            request_info.add_request_options(request_configuration.options)
        request_info.set_content_from_parsable(self.request_adapter, "application/json", body)
        return request_info
    
    @property
    def site_lists(self) -> site_lists_request_builder.SiteListsRequestBuilder:
        """
        Provides operations to manage the siteLists property of the microsoft.graph.internetExplorerMode entity.
        """
        from .site_lists import site_lists_request_builder

        return site_lists_request_builder.SiteListsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @dataclass
    class InternetExplorerModeRequestBuilderDeleteRequestConfiguration():
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        # Request headers
        headers: Optional[Dict[str, Union[str, List[str]]]] = None

        # Request options
        options: Optional[List[RequestOption]] = None

    
    @dataclass
    class InternetExplorerModeRequestBuilderGetQueryParameters():
        """
        Get internetExplorerMode from admin
        """
        def get_query_parameter(self,original_name: Optional[str] = None) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            Args:
                originalName: The original query parameter name in the class.
            Returns: str
            """
            if original_name is None:
                raise Exception("original_name cannot be undefined")
            if original_name == "expand":
                return "%24expand"
            if original_name == "select":
                return "%24select"
            return original_name
        
        # Expand related entities
        expand: Optional[List[str]] = None

        # Select properties to be returned
        select: Optional[List[str]] = None

    
    @dataclass
    class InternetExplorerModeRequestBuilderGetRequestConfiguration():
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        # Request headers
        headers: Optional[Dict[str, Union[str, List[str]]]] = None

        # Request options
        options: Optional[List[RequestOption]] = None

        # Request query parameters
        query_parameters: Optional[InternetExplorerModeRequestBuilder.InternetExplorerModeRequestBuilderGetQueryParameters] = None

    
    @dataclass
    class InternetExplorerModeRequestBuilderPatchRequestConfiguration():
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        # Request headers
        headers: Optional[Dict[str, Union[str, List[str]]]] = None

        # Request options
        options: Optional[List[RequestOption]] = None

    

